"""Asynchronous Tariff API client."""

from __future__ import annotations

from neo_tariff._http import AsyncHttpTransport
from neo_tariff.resources import (
    AsyncCompareResource,
    AsyncContextResource,
    AsyncRatesResource,
    AsyncSearchResource,
    AsyncVersionsResource,
)

DEFAULT_BASE_URL = "https://tariff-data.enterprise-neo.com"


class AsyncNeoTariff:
    """Asynchronous client for the Neo Tariff API.

    Usage::

        from neo_tariff import AsyncNeoTariff

        client = AsyncNeoTariff(api_key="ntf_...")
        result = await client.rates.evaluate_entry(
            hts_code="7208.10.15",
            country_of_origin="CN",
            cost=10000,
            qty=1000,
        )
        print(result.data)

    Or as an async context manager::

        async with AsyncNeoTariff(api_key="ntf_...") as client:
            result = await client.rates.evaluate_entry(...)
    """

    rates: AsyncRatesResource
    search: AsyncSearchResource
    context: AsyncContextResource
    compare: AsyncCompareResource
    versions: AsyncVersionsResource

    def __init__(
        self,
        *,
        api_key: str,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = 30.0,
        max_retries: int = 2,
    ) -> None:
        self._http = AsyncHttpTransport(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
        )
        self.rates = AsyncRatesResource(self._http)
        self.search = AsyncSearchResource(self._http)
        self.context = AsyncContextResource(self._http)
        self.compare = AsyncCompareResource(self._http)
        self.versions = AsyncVersionsResource(self._http)

    @property
    def with_raw_response(self) -> AsyncNeoTariff:
        """Returns a client variant whose resource methods return RawResponse.

        Uses transport swapping: the same async resource classes are reused,
        but wired with a transport that calls ``request_raw()`` instead of
        ``request()`` / ``request_typed()``.  Return type annotations on
        resource methods will technically be inaccurate — they say
        ``APIResponse`` but actually return ``RawResponse``.  This is an
        intentional trade-off for a power-user escape hatch.
        """
        from neo_tariff._raw import RawAsyncHttpTransport

        raw_client = AsyncNeoTariff.__new__(AsyncNeoTariff)
        raw_transport = RawAsyncHttpTransport(self._http)
        raw_client._http = raw_transport
        raw_client.rates = AsyncRatesResource(raw_transport)
        raw_client.search = AsyncSearchResource(raw_transport)
        raw_client.context = AsyncContextResource(raw_transport)
        raw_client.compare = AsyncCompareResource(raw_transport)
        raw_client.versions = AsyncVersionsResource(raw_transport)
        return raw_client

    async def close(self) -> None:
        """Close the underlying HTTP connection pool."""
        await self._http.close()

    async def __aenter__(self) -> AsyncNeoTariff:
        return self

    async def __aexit__(self, *args: object) -> None:
        await self.close()



"""Base resource classes for sync and async transports."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from neo_tariff._http import AsyncHttpTransport, HttpTransport


class SyncResource:
    """Base class for synchronous resource namespaces."""

    def __init__(self, http: HttpTransport) -> None:
        self._http = http


class AsyncResource:
    """Base class for asynchronous resource namespaces."""

    def __init__(self, http: AsyncHttpTransport) -> None:
        self._http = http



"""Synchronous Tariff API client."""

from __future__ import annotations

from neo_tariff._http import HttpTransport
from neo_tariff.resources import (
    CompareResource,
    ContextResource,
    RatesResource,
    SearchResource,
    VersionsResource,
)

DEFAULT_BASE_URL = "https://tariff-data.enterprise-neo.com"


class NeoTariff:
    """Synchronous client for the Neo Tariff API.

    Usage::

        from neo_tariff import NeoTariff

        client = NeoTariff(api_key="ntf_...")
        result = client.rates.evaluate_entry(
            hts_code="7208.10.15",
            country_of_origin="CN",
            cost=10000,
            qty=1000,
        )
        print(result.data)

    Or as a context manager::

        with NeoTariff(api_key="ntf_...") as client:
            result = client.rates.evaluate_entry(...)
    """

    rates: RatesResource
    search: SearchResource
    context: ContextResource
    compare: CompareResource
    versions: VersionsResource

    def __init__(
        self,
        *,
        api_key: str,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = 30.0,
        max_retries: int = 2,
    ) -> None:
        self._http = HttpTransport(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
        )
        self.rates = RatesResource(self._http)
        self.search = SearchResource(self._http)
        self.context = ContextResource(self._http)
        self.compare = CompareResource(self._http)
        self.versions = VersionsResource(self._http)

    @property
    def with_raw_response(self) -> NeoTariff:
        """Returns a client variant whose resource methods return RawResponse.

        Uses transport swapping: the same resource classes are reused, but
        wired with a transport that calls ``request_raw()`` instead of
        ``request()`` / ``request_typed()``.  Return type annotations on
        resource methods will technically be inaccurate — they say
        ``APIResponse`` but actually return ``RawResponse``.  This is an
        intentional trade-off for a power-user escape hatch.
        """
        from neo_tariff._raw import RawHttpTransport

        raw_client = NeoTariff.__new__(NeoTariff)
        raw_transport = RawHttpTransport(self._http)
        raw_client._http = raw_transport
        raw_client.rates = RatesResource(raw_transport)
        raw_client.search = SearchResource(raw_transport)
        raw_client.context = ContextResource(raw_transport)
        raw_client.compare = CompareResource(raw_transport)
        raw_client.versions = VersionsResource(raw_transport)
        return raw_client

    def close(self) -> None:
        """Close the underlying HTTP connection pool."""
        self._http.close()

    def __enter__(self) -> NeoTariff:
        return self

    def __exit__(self, *args: object) -> None:
        self.close()
